//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// mmc.h
//
// supported MMC interfaces
//
// $Id: $
//

#ifndef _MMC_H_
#define _MMC_H_

#include "typedefs.h"
#include "nes6502.h"
#include "unit.h"
#include "libsnss/libsnss.h"

#define  SRAM_BANK_SIZE       0x0400
#define  VRAM_BANK_SIZE       0x2000


typedef enum
{
   MIRROR_HORIZ      = 0,
   MIRROR_VERT       = 1,
   MIRROR_ONE_LOW    = 2,
   MIRROR_ONE_HIGH   = 3,
   MIRROR_FOUR       = 4,
   MIRROR_MAPPER     = 5
} mirror_t;

struct mapperInfo
{
   // memory
   uint8    *mpPRG;
   uint8    *mpCHR;
   uint8    *mpSRAM;
   uint8    *mpVRAM;

   // bank information
   uint32   mNumPRG;
   uint32   mNumCHR;

   uint32   mNumSRAM;
   uint32   mNumVRAM;

   // board hardwired mirroring
   mirror_t mMirror;

   // VS unisystem palette
   uint8    *mpPalette;
   
   int      mMapperID;
};

class machine;
// interface to MMC controller functions
class mmc : public unit
{
public:
   mmc(machine *pMachine, mapperInfo *pInfo);
   virtual ~mmc();

   // SNSS state
   virtual void  getState(SnssMapperBlock *pState);
   virtual void  setState(SnssMapperBlock *pState);

   // reset procedure
   virtual void  reset(bool hard_reset);

   // vblank / hblank callbacks
   virtual void  vblank();
   virtual void  hblank(bool in_vblank);

protected:
   enum
   {
      CART_LASTBANK  = -1
   };

   const char *mpName;

   mapperInfo mInfo;
   void prgbank(int size, uint32 address, int bank);
   void chrbank(int size, uint32 address, int bank);
};

// return a pointer to a new MMC interface, or
// NULL if it is not supported
extern mmc *mmc_create(machine *pMachine, mapperInfo *pInfo);

#endif // !_MMC_H_
